#include "kaphorism.h"
#include <stdio.h>
#include <kconfigbackend.h>


kaphorism_config::kaphorism_config(QString path) : KSimpleConfig(path,true)
{
  setLanguage(QString language);
}

void kaphorism_config::setLanguage(QString lang)
{
QString grp=group();
  setLocale();
  setGroup("language");
  QCString loc(readEntry("locale",locale()).utf8());
  aLocaleString=loc;
  backEnd->setLocaleString(aLocaleString);
  QString a=aLocaleString;
  printf("%s\n",a.latin1());
  setGroup(grp);
}

