/****************************************************************************
** Form interface generated from reading ui file './prefdialogview.ui'
**
** Created: Sun Mar 31 22:48:52 2002
**      by:  The User Interface Compiler (uic)
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/
#ifndef PREFSDIALOGDATA_H
#define PREFSDIALOGDATA_H

#include <qvariant.h>
#include <kdialogbase.h>
class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class KIntNumInput;
class KURLLabel;
class KURLRequester;
class QCheckBox;
class QLabel;
class QLineEdit;
class QPushButton;
class QSpinBox;
class QToolButton;

class prefsDialogData : public KDialogBase
{ 
    Q_OBJECT

public:
    prefsDialogData( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~prefsDialogData();

    KIntNumInput* m_updateInterval;
    QCheckBox* m_enableLog;
    KURLRequester* m_logFile;
    QPushButton* fontButton;
    QLabel* samplefont;
    QLabel* m_labelLogFile;
    QCheckBox* m_offlineMode;
    QCheckBox* m_smallView;
    QLabel* locationLabel;
    QLineEdit* m_reportLocation;
    KURLLabel* ICAOlink;
};

#endif // PREFSDIALOGDATA_H
